# Generated by Django 4.2.1 on 2023-07-07 19:32

from django.db import migrations
import edc_lab.managers
import edc_visit_tracking.managers


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0051_alter_bloodresultsfbc_options_and_more"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="bloodresultsfbc",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultsglu",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultshba1c",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultsins",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultslft",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultslipid",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="bloodresultsrft",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="cd4result",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="clinicalreview",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="clinicalreviewbaseline",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="complicationsbaseline",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="complicationsfollowup",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="concomitantmedication",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="dminitialreview",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="dmmedicationadherence",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="dmreview",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="drugrefilldm",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="drugrefillhiv",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="drugrefillhtn",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="eq5d3l",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="familyhistory",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="glucose",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="hba1cresult",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomics",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomicsassets",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomicshouseholdhead",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomicsincome",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomicspatient",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="healtheconomicsproperty",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="hivinitialreview",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="hivmedicationadherence",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="hivreview",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="htninitialreview",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="htnmedicationadherence",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="htnreview",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="icecapa",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="investigations",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="malariatest",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="medications",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="nextappointment",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="otherbaselinedata",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="socialharms",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="subjectrequisition",
            managers=[
                ("objects", edc_lab.managers.RequisitionManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="subjectvisitmissed",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="urinedipsticktest",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="urinepregnancy",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="viralloadresult",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="vitals",
            managers=[
                ("objects", edc_visit_tracking.managers.CrfModelManager()),
                ("on_site", edc_visit_tracking.managers.CrfCurrentSiteManager()),
            ],
        ),
    ]



# Generated by Django 4.2.1 on 2023-07-05 03:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0043_auto_20230705_0529"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="This question is asked to confirm you are logged in to the correct site.",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="intecomm_screening.site",
                verbose_name="Which site is this?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="patient_log_identifier",
            field=models.CharField(
                db_index=True, max_length=50, verbose_name="Patient log identifier"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="intecomm_screening.site",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="site",
            field=models.ForeignKey(
                help_text="This question is asked to confirm you are logged in to the correct site.",
                on_delete=django.db.models.deletion.PROTECT,
                to="intecomm_screening.site",
                verbose_name="Which site is this?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="patient_log_identifier",
            field=models.CharField(
                max_length=50, unique=True, verbose_name="Patient log identifier"
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="intecomm_screening.site",
            ),
        ),
    ]



# Generated by Django 4.2.7 on 2023-11-29 20:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0126_feat_683_land_area_units"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectvisitmissed",
            name="appt_date",
            field=models.DateField(
                blank=True,
                help_text="Should fall on an Integrated clinic day",
                null=True,
                verbose_name="Next scheduled routine/facility appointment",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisitmissed",
            name="appt_date",
            field=models.DateField(
                blank=True,
                help_text="Should fall on an Integrated clinic day",
                null=True,
                verbose_name="Next scheduled routine/facility appointment",
            ),
        ),
    ]



from datetime import date

from django.db.models import Count, Min
from edc_dx.diagnoses import (
    ClinicalReviewBaselineRequired,
    Diagnoses,
    InitialReviewRequired,
)
from edc_pdutils.model_to_dataframe import ModelToDataframe
from edc_utils import get_utcnow
from tqdm import tqdm

from intecomm_subject.models import SubjectVisit
from intecomm_subject.utils import recalculate_dx_calculated_date

from ..models import Diagnoses as DiagnosesModel


def export_diagnoses_model(path: str | None = None):
    m = ModelToDataframe(model="intecomm_reports.diagnoses")
    df = m.dataframe
    df["hiv_baseline"] = 0
    df["htn_baseline"] = 0
    df["dm_baseline"] = 0
    df.loc[(df["hiv_dx_date"] < df["baseline_date"]) & (df["hiv"] == 1), "hiv_baseline"] = 1
    df.loc[(df["htn_dx_date"] < df["baseline_date"]) & (df["htn"] == 1), "htn_baseline"] = 1
    df.loc[(df["dm_dx_date"] < df["baseline_date"]) & (df["dm"] == 1), "dm_baseline"] = 1
    df["condition_count"] = df[["hiv_baseline", "htn_baseline", "dm_baseline"]].sum(axis=1)
    if path:
        df.to_csv(
            path,
            columns=[
                "subject_identifier",
                "baseline_date",
                "hiv",
                "htn",
                "dm",
                "hiv_dx_date",
                "htn_dx_date",
                "dm_dx_date",
                "hiv_baseline",
                "htn_baseline",
                "dm_baseline",
                "condition_count",
            ],
            index=False,
        )
    return df


def update_diagnoses_model(delete_all: bool | None = None, path: str = None):
    """Calculate the diagnoses of each participant who has completed
    a baseline visit.

    To be considered having completed the baseline visit, the
    ClinicalReviewBaseline and the corresponding IntialReview forms
    must be complete.

    Protocol requires participants be diagnosed at least 6m ago and
    stable in care for at least 6 months. When querying the InitialReview
    forms, the diagnosis date must be at least 182 days prior to the
    baseline visit date.
    """
    errors = []
    if delete_all:
        DiagnosesModel.objects.all().delete()
    qs = (
        SubjectVisit.objects.values("subject_identifier", "site_id")
        .annotate(
            Count("subject_identifier", distinct=True),
            Count("site_id", distinct=True),
        )
        .order_by("subject_identifier")
        .annotate(Min("report_datetime"))
    )
    total = qs.count()
    for subject_visit in tqdm(qs, total=total):
        subject_identifier = subject_visit.get("subject_identifier")
        site_id = subject_visit.get("site_id")
        baseline_date = subject_visit.get("report_datetime__min").date()
        try:
            dx = Diagnoses(
                subject_identifier=subject_identifier,
                report_datetime=get_utcnow(),
                lte=True,
            )
        except ClinicalReviewBaselineRequired as e:
            errors.append(f"{subject_identifier}: {e}")
        else:
            errors.extend(
                update_diagnoses_instance(dx, subject_identifier, site_id, baseline_date)
            )
    for err in errors:
        print(err)


def update_diagnoses_instance(
    dx: Diagnoses,
    subject_identifier: str = None,
    site_id: str = None,
    baseline_date: date = None,
) -> list:
    """Updates for a single condition.

    Called by `update_diagnoses_model`.
    """
    errors = []
    obj, _ = DiagnosesModel.objects.get_or_create(
        subject_identifier=subject_identifier, site_id=site_id
    )
    obj.baseline_date = baseline_date
    try:
        initial_reviews = dx.initial_reviews
    except InitialReviewRequired:
        errors.append(f"{obj.subject_identifier}: initial_reviews not found!")
    else:
        for prefix in dx.diagnosis_labels:
            if initial_review := initial_reviews.get(prefix):
                recalculate_dx_calculated_date(
                    models={prefix: initial_review.__class__},
                    subject_identifier=obj.subject_identifier,
                )
                initial_review.refresh_from_db()
                if dx_date := (
                    getattr(initial_review, "dx_date")
                    or getattr(initial_review, "dx_calculated_date")
                ):
                    setattr(obj, prefix, True)
                    setattr(obj, f"{prefix}_dx_date", dx_date)
                    dx_days = (dx_date - obj.baseline_date).days
                    setattr(obj, f"{prefix}_dx_days", dx_days)
                else:
                    errors.append(
                        f"{obj.subject_identifier}: "
                        f"`dx_calculated_date` is None. Got {prefix}: "
                        f"{getattr(initial_review, 'dx_date')}"
                        f"{getattr(initial_review, 'dx_ago')}"
                    )
                    obj.comment = ",".join(errors)
                    setattr(obj, prefix, False)
                    setattr(obj, f"{prefix}_dx_date", None)
                    setattr(obj, f"{prefix}_dx_days", None)

    obj.save()
    return errors



from django import forms
from edc_form_validators import FormValidator

from ..models import FamilyHistory
from .mixins import CrfModelFormMixin


class FamilyHistoryFormValidator(FormValidator):
    pass


class FamilyHistoryForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = FamilyHistoryFormValidator

    class Meta:
        model = FamilyHistory
        fields = "__all__"



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
        select B1.* from (
        select hivreview.*, dx.baseline_date, TIMESTAMPDIFF(MONTH, baseline_date, vl_date) as m
        from (
        select subject_identifier, crf.site_id, vl as vl_value,
        case when drawn_date is null then cast(crf.report_datetime as DATE) else drawn_date end as vl_date
        from intecomm_subject_hivinitialreview as crf
        left join intecomm_subject_subjectvisit as v on v.id=crf.subject_visit_id
        where vl is not null
        UNION
        select subject_identifier, crf.site_id, vl as vl_value,
        case when drawn_date is null then cast(crf.report_datetime as DATE) else drawn_date end as vl_date
        from intecomm_subject_hivreview as crf
        left join intecomm_subject_subjectvisit as v on v.id=crf.subject_visit_id
        ) as hivreview
        left join intecomm_reports_diagnoses as dx on dx.subject_identifier=hivreview.subject_identifier
        where vl_value is not null) as B1
        """  # noqa

    sql_view = SqlViewGenerator(
        report_model="intecomm_reports.vl_view",
        ordering=["site_id", "subject_identifier"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 4.2.11 on 2024-07-02 04:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0006_auto_20240702_0406"),
    ]

    operations = []



import datetime as dt
import os
from zoneinfo import ZoneInfo

import time_machine
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, override_settings
from edc_appointment.models import Appointment, AppointmentType
from edc_constants.constants import CLINIC, COMMUNITY, HTN
from edc_metadata import REQUIRED
from edc_metadata.models import CrfMetadata
from edc_randomization.site_randomizers import site_randomizers
from edc_utils import get_utcnow
from edc_visit_tracking.constants import SCHEDULED
from intecomm_rando.constants import COMMUNITY_ARM
from intecomm_rando.randomizers import Randomizer
from intecomm_rando.utils import get_assignment_for_subject

from intecomm_screening.tests.intecomm_test_case_mixin import IntecommTestCaseMixin
from intecomm_subject.models import SubjectVisit

utc_tz = ZoneInfo("UTC")


@override_settings(
    EDC_RANDOMIZATION_LIST_PATH=os.path.join(
        settings.BASE_DIR, "intecomm_subject", "tests", "etc", "community_only"
    )
)
@time_machine.travel(dt.datetime(2019, 5, 11, 8, 00, tzinfo=utc_tz))
class TestLocationUpdate(IntecommTestCaseMixin, TestCase):
    patient_group = None
    import_randomization_list = False
    sid_count_for_tests = 2

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        site_randomizers._registry = {}
        Randomizer.randomizationlist_folder = settings.EDC_RANDOMIZATION_LIST_PATH
        site_randomizers.register(Randomizer)
        randomizer_cls = site_randomizers.get("default")
        randomizer_cls.import_list(verbose=False, sid_count_for_tests=cls.sid_count_for_tests)
        cls.patient_group = cls.get_randomized_patient_group(
            report_datetime=get_utcnow() - relativedelta(days=10), group_name="BRANDX"
        )

    def setUp(self):
        self.assertTrue(self.patient_group.randomized)

    def test_crf_metadata(self):
        patient_log = self.patient_group.patients.filter(conditions__name__in=[HTN]).first()
        self.get_subject_visit(subject_identifier=patient_log.subject_identifier)
        subject_visit = self.get_subject_visit(
            subject_identifier=patient_log.subject_identifier, visit_code="1010"
        )

        self.assertEqual(
            get_assignment_for_subject(subject_visit.subject_identifier), COMMUNITY_ARM
        )
        appointment = Appointment.objects.get(
            subject_identifier=subject_visit.subject_identifier, visit_code="1020"
        )
        appointment.appt_type = AppointmentType.objects.get(name=COMMUNITY)
        appointment.save()
        subject_visit = SubjectVisit.objects.create(
            appointment=appointment,
            report_datetime=appointment.appt_datetime,
            reason=SCHEDULED,
        )
        try:
            CrfMetadata.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                visit_code=subject_visit.visit_code,
                model="intecomm_subject.locationupdate",
                entry_status=REQUIRED,
            )
        except ObjectDoesNotExist:
            pass
        else:
            self.fail("CrfMetadata for `intecomm_subject.locationupdate` unexpectedly exists")

        appointment.appt_type = AppointmentType.objects.get(name=CLINIC)  # facility
        appointment.save()
        subject_visit.save()
        try:
            CrfMetadata.objects.get(
                subject_identifier=subject_visit.subject_identifier,
                visit_code=subject_visit.visit_code,
                model="intecomm_subject.locationupdate",
                entry_status=REQUIRED,
            )
        except ObjectDoesNotExist:
            self.fail(
                "CrfMetadata for `intecomm_subject.locationupdate` unexpectedly does not exist"
            )



# Generated by Django 4.1.7 on 2023-03-31 02:19

from django.db import migrations, models
import edc_model.models.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_lists", "0004_rxmodificationreasons_rxmodifications_and_more"),
        ("intecomm_subject", "0015_alter_bloodresultsfbc_results_abnormal_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="clinicalreview",
            name="reason_other",
        ),
        migrations.RemoveField(
            model_name="historicalclinicalreview",
            name="reason_other",
        ),
        migrations.AlterField(
            model_name="clinicalreview",
            name="dm_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="As of today, was the patient <u>newly</u> diagnosed with diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreview",
            name="dm_reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreview",
            name="hiv_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="As of today, was the patient <u>newly</u> diagnosed with HIV infection?",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreview",
            name="hiv_reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreview",
            name="htn_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="As of today, was the patient <u>newly</u> diagnosed with hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreview",
            name="htn_reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefilldm",
            name="modifications",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodifications",
                verbose_name="Which changes occurred?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefilldm",
            name="modifications_reason",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodificationreasons",
                verbose_name="Why did the patient’s previous prescription change?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhiv",
            name="modifications",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodifications",
                verbose_name="Which changes occurred?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhiv",
            name="modifications_reason",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodificationreasons",
                verbose_name="Why did the patient’s previous prescription change?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhtn",
            name="modifications",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodifications",
                verbose_name="Which changes occurred?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhtn",
            name="modifications_reason",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodificationreasons",
                verbose_name="Why did the patient’s previous prescription change?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreview",
            name="dm_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="As of today, was the patient <u>newly</u> diagnosed with diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreview",
            name="dm_reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreview",
            name="hiv_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="As of today, was the patient <u>newly</u> diagnosed with HIV infection?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreview",
            name="hiv_reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreview",
            name="htn_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="As of today, was the patient <u>newly</u> diagnosed with hypertension?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreview",
            name="htn_reason_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



# Generated by Django 5.0 on 2023-12-16 15:06

import django.db.models.deletion
import django.db.models.manager
import edc_sites.managers
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_screening",
            "0065_patientlogreportprinthistory_intecomm_sc_modifie_891a97_idx_and_more",
        ),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="patientlogreportprinthistory",
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
                ("objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AddField(
            model_name="patientlogreportprinthistory",
            name="site",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
    ]



from edc_lab_panel.panels import (
    blood_glucose_panel,
    cd4_panel,
    fbc_panel,
    hba1c_panel,
    lft_panel,
    lipids_panel,
    rft_panel,
    vl_panel,
)
from edc_visit_schedule.visit import FormsCollection, Requisition

requisitions_prn = FormsCollection(
    Requisition(show_order=200, panel=blood_glucose_panel, required=True, additional=False),
    Requisition(show_order=220, panel=hba1c_panel, required=True, additional=False),
    Requisition(show_order=230, panel=rft_panel, required=True, additional=False),
    Requisition(show_order=240, panel=lft_panel, required=True, additional=False),
    Requisition(show_order=250, panel=fbc_panel, required=True, additional=False),
    Requisition(show_order=260, panel=lipids_panel, required=True, additional=False),
    Requisition(show_order=280, panel=cd4_panel, required=True, additional=False),
    Requisition(show_order=300, panel=vl_panel, required=True, additional=False),
    name="requisitions_prn",
)

requisitions_unscheduled = FormsCollection(name="requisitions_unscheduled")
requisitions_d1 = FormsCollection(name="requisitions_day1")
requisitions_followup = FormsCollection(name="requisitions_followup")



# Generated by Django 4.2.3 on 2023-08-03 15:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0084_alter_healtheconomicsassets_external_wall_material_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="external_wall_material",
            field=models.CharField(
                choices=[
                    ("1", "Thatch, Straw"),
                    ("2", "Mud and poles"),
                    ("3", "Timber"),
                    ("OPTION_RETIRED", "Un-burnt bricks"),
                    ("5", "Bricks with mud"),
                    ("6", "Bricks with cement"),
                    ("7", "Cement blocks"),
                    ("8", "Stone"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the major construction material of the external wall?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="iron",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], max_length=15, verbose_name="Flat iron"
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="external_wall_material",
            field=models.CharField(
                choices=[
                    ("1", "Thatch, Straw"),
                    ("2", "Mud and poles"),
                    ("3", "Timber"),
                    ("OPTION_RETIRED", "Un-burnt bricks"),
                    ("5", "Bricks with mud"),
                    ("6", "Bricks with cement"),
                    ("7", "Cement blocks"),
                    ("8", "Stone"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the major construction material of the external wall?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="iron",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], max_length=15, verbose_name="Flat iron"
            ),
        ),
    ]



# Generated by Django 4.2.3 on 2023-07-17 14:37

import django.core.validators
from django.db import migrations, models
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0056_alter_drugrefilldm_rx_days_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalotherbaselinedata",
            old_name="smoker_current_duration",
            new_name="smoker_duration",
        ),
        migrations.RenameField(
            model_name="historicalotherbaselinedata",
            old_name="smoker_current_duration_estimated_date",
            new_name="smoker_duration_estimated_date",
        ),
        migrations.RenameField(
            model_name="otherbaselinedata",
            old_name="smoker_current_duration",
            new_name="smoker_duration",
        ),
        migrations.RenameField(
            model_name="otherbaselinedata",
            old_name="smoker_current_duration_estimated_date",
            new_name="smoker_duration_estimated_date",
        ),
        migrations.RemoveField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_citizen",
        ),
        migrations.RemoveField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_citizen",
        ),
        migrations.AddField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                help_text="... other type of employment",
                max_length=100,
                null=True,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                help_text="... other type of employment",
                max_length=100,
                null=True,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hh_count",
            field=models.IntegerField(
                help_text="Persons",
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(25),
                ],
                verbose_name="What is the total number of people who live in your household?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hh_minors_count",
            field=models.IntegerField(
                help_text="Persons",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(25),
                ],
                verbose_name="What is the total number of people 14 years or under who live in your household?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="relationship_to_hoh",
            field=models.CharField(
                choices=[
                    ("WIFE_HUSBAND", "Wife/Husband"),
                    ("SON_DAUGHTER", "Son/Daughter"),
                    ("SON_DAUGHTERINLAW", "Son/Daughter-in-law"),
                    ("GRANDCHILD", "Grandchild"),
                    ("PARENT", "Parent"),
                    ("PARENTINLAW", "Parent-in-law"),
                    ("BROTHER_SISTER", "Brother/Sister"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="What is your relationship to the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hh_count",
            field=models.IntegerField(
                help_text="Persons",
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(25),
                ],
                verbose_name="What is the total number of people who live in your household?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hh_minors_count",
            field=models.IntegerField(
                help_text="Persons",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(25),
                ],
                verbose_name="What is the total number of people 14 years or under who live in your household?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="relationship_to_hoh",
            field=models.CharField(
                choices=[
                    ("WIFE_HUSBAND", "Wife/Husband"),
                    ("SON_DAUGHTER", "Son/Daughter"),
                    ("SON_DAUGHTERINLAW", "Son/Daughter-in-law"),
                    ("GRANDCHILD", "Grandchild"),
                    ("PARENT", "Parent"),
                    ("PARENTINLAW", "Parent-in-law"),
                    ("BROTHER_SISTER", "Brother/Sister"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don’t know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="What is your relationship to the household head?",
            ),
        ),
    ]



from django import forms
from intecomm_form_validators.subject import OtherBaselineDataFormValidator

from ..models import OtherBaselineData
from .mixins import CrfModelFormMixin


class OtherBaselineDataForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = OtherBaselineDataFormValidator

    class Meta:
        model = OtherBaselineData
        fields = "__all__"



from django.db import models
from edc_action_item.models.action_model_mixin import ActionModelMixin
from edc_constants.choices import YES_NO
from edc_constants.constants import OTHER
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_ltfu.constants import LTFU_ACTION
from edc_model.models import BaseUuidModel, OtherCharField
from edc_sites.model_mixins import SiteModelMixin
from edc_utils.date import get_utcnow

LOSS_CHOICES = (
    ("unknown_address", "Changed to an unknown address"),
    ("never_returned", "Did not return despite reminders"),
    ("bad_contact_details", "Inaccurate contact details"),
    (OTHER, "Other, please specify ..."),
)


class LossToFollowup(
    SiteModelMixin,
    ActionModelMixin,
    NonUniqueSubjectIdentifierFieldMixin,
    BaseUuidModel,
):
    action_name = LTFU_ACTION

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    last_seen_datetime = models.DateField(verbose_name="Date participant last seen")

    last_missed_visit_datetime = models.DateField(
        verbose_name="Date of last missed visit report submitted",
        null=True,
        blank=False,
    )

    home_visited = models.CharField(
        verbose_name="Has a home visit been made", max_length=15, choices=YES_NO
    )

    home_visit_detail = models.TextField(
        verbose_name="If YES, provide any further details of the home visit",
        null=True,
        blank=False,
    )

    loss_category = models.CharField(
        verbose_name="Category of loss to follow up",
        max_length=25,
        choices=LOSS_CHOICES,
    )

    loss_category_other = OtherCharField()

    comment = models.TextField(
        verbose_name=(
            "Please give details of the circumstances that have led to this decision."
        ),
        null=True,
        blank=False,
    )

    class Meta(ActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Loss to Follow Up"
        verbose_name_plural = "Loss to Follow Up"
        indexes = (
            NonUniqueSubjectIdentifierFieldMixin.Meta.indexes
            + ActionModelMixin.Meta.indexes
            + BaseUuidModel.Meta.indexes
            + [models.Index(fields=["subject_identifier", "action_identifier", "site", "id"])]
        )



# Generated by Django 4.2.11 on 2024-06-18 04:27
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from django.db.migrations import RunPython
from edc_constants.constants import FEMALE, MALE
from edc_utils import get_utcnow


def update_gender(apps, schema_editor):
    """Update gender from 'M' to 'F' for participant added to patient
    log (and subsequently screened and consented) with the wrong gender.

    See also:
        Redmine ticket: #812
        Protocol incident: F2DV-PMTV
    """
    for model in ["intecomm_consent.subjectconsent", "edc_registration.registeredsubject"]:
        model_cls = apps.get_model(model)
        try:
            obj = model_cls.objects.get(
                subject_identifier="107-110-0045-4",
                screening_identifier="SU2VXC66",
                site_id=110,
                gender=MALE,
            )
        except ObjectDoesNotExist:
            print("Matching object does not exist. Skipping ...")
            break
        else:
            print(
                f"\n  * Updating {model_cls.__name__} gender for {obj.subject_identifier=} "
                f"({obj.screening_identifier=}) from '{MALE}' to '{FEMALE}' ..."
            )

            obj.modified = get_utcnow()
            obj.user_modified = __name__ if len(__name__) <= 50 else f"{__name__[:46]} ..."
            obj.gender = FEMALE
            obj.save_base(
                update_fields=[
                    "gender",
                    "modified",
                    "user_modified",
                ]
            )

            print("  * Updated.")
    print("Done.")


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_consent", "0026_data_fix_810_gender_update"),
        ("intecomm_screening", "0073_data_fix_812_gender_update"),
    ]

    operations = [RunPython(update_gender)]



# Generated by Django 4.2.10 on 2024-02-22 05:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "intecomm_subject",
            "0145_rename_facility_visit_alt_careseekinga_missed_activities_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="careseekingb",
            old_name="seek_inpatient_visits",
            new_name="inpatient_days",
        ),
        migrations.RenameField(
            model_name="careseekingb",
            old_name="seek_outpatient_visits",
            new_name="outpatient_visits",
        ),
        migrations.RenameField(
            model_name="historicalcareseekingb",
            old_name="seek_inpatient_visits",
            new_name="inpatient_days",
        ),
        migrations.RenameField(
            model_name="historicalcareseekingb",
            old_name="seek_outpatient_visits",
            new_name="outpatient_visits",
        ),
    ]



from .assets_admin import HealthEconomicsAssetsAdmin
from .careseeking_a_admin import CareseekingAAdmin
from .careseeking_b_admin import CareseekingBAdmin
from .household_head_admin import HealthEconomicsHouseholdHeadAdmin
from .income_admin import HealthEconomicsIncomeAdmin
from .patient_admin import HealthEconomicsPatientAdmin
from .property_admin import HealthEconomicsPropertyAdmin



from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import UniqueConstraint
from django.utils.html import format_html
from django_crypto_fields.fields import EncryptedCharField
from edc_constants.choices import PREG_YES_NO_NA, SELECTION_METHOD, YES_NO, YES_NO_NA
from edc_constants.constants import NO, NOT_APPLICABLE, PURPOSIVELY_SELECTED, TBD
from edc_model.models import BaseUuidModel, DurationYMDField, NameFieldsModelMixin
from edc_screening.model_mixins import EligibilityModelMixin, ScreeningModelMixin
from edc_screening.screening_identifier import (
    ScreeningIdentifier as BaseScreeningIdentifier,
)
from edc_vitals.model_mixins import BloodPressureModelMixin
from intecomm_eligibility import ScreeningEligibility

from intecomm_consent.consents import cdef_tz_v1

from .patient_log import PatientLog
from .proxy_models import Site


class SubjectScreeningError(Exception):
    pass


class ScreeningIdentifier(BaseScreeningIdentifier):
    template = "S{random_string}"


class SubjectScreening(
    EligibilityModelMixin,
    BloodPressureModelMixin,
    ScreeningModelMixin,
    NameFieldsModelMixin,
    BaseUuidModel,
):
    identifier_cls = ScreeningIdentifier
    eligibility_cls = ScreeningEligibility
    consent_definitions = [cdef_tz_v1]

    # declare `site` explicitly as editable by user
    # (instead of using modelmixin)
    site = models.ForeignKey(Site, on_delete=models.PROTECT, null=True, related_name="+")

    screening_identifier = models.CharField(
        verbose_name="Screening ID",
        max_length=50,
        blank=True,
        unique=True,
    )

    patient_log_identifier = models.CharField(
        verbose_name="Patient log identifier",
        max_length=50,
        unique=True,
    )

    patient_log = models.OneToOneField(
        PatientLog, on_delete=models.PROTECT, null=True, blank=True
    )

    selection_method = models.CharField(
        verbose_name="How was the patient selected for screening?",
        max_length=25,
        choices=SELECTION_METHOD,
        default=PURPOSIVELY_SELECTED,
    )

    hospital_identifier = EncryptedCharField(unique=True, blank=False)

    lives_nearby = models.CharField(
        verbose_name=(
            "Does the patient live within the catchment population of the health facility"
        ),
        max_length=15,
        choices=YES_NO,
    )

    staying_nearby_6 = models.CharField(
        verbose_name=(
            "Is the patient planning to remain in this catchment area for at least 6 months"
        ),
        max_length=15,
        choices=YES_NO,
    )

    in_care_6m = models.CharField(
        verbose_name=(
            "Has the patient been in regular care at this health facility for 6 months or more"
        ),
        max_length=15,
        choices=YES_NO,
        help_text="for 6m or more at the health facility from which they are being recruited",
    )

    in_care_duration = DurationYMDField(
        verbose_name=(
            "If 'Yes', estimate how long the patient been in regular care at this "
            "health facility"
        ),
        null=True,
        blank=True,
    )

    hiv_dx = models.CharField(
        verbose_name="Has the patient previously been diagnosed with HIV",
        max_length=15,
        choices=YES_NO,
    )

    hiv_dx_6m = models.CharField(
        verbose_name="Was the initial diagnosis made more than 6 months ago",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    hiv_dx_ago = DurationYMDField(
        verbose_name="If 'Yes', estimate how long ago the initial diagnosis was made",
        null=True,
        blank=True,
    )

    art_unchanged_3m = models.CharField(
        verbose_name=format_html(
            "Has the patient been on the same anti-retroviral therapy for at least 3 months"
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "Meaning no modifications to the type of medication and dose for at least 3 months"
        ),
    )

    art_stable = models.CharField(
        verbose_name="Is the patient considered to be stable on treatment ",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=(
            "considered by the clinical team not to have any complications/co-infections "
            "or that these are well managed."
        ),
    )

    art_adherent = models.CharField(
        verbose_name=(
            "Is the patient considered to be adherent to treatment over the last 6 months"
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="in regular attendance for care",
    )

    dm_dx = models.CharField(
        verbose_name="Has the patient previously been diagnosed with Diabetes",
        max_length=15,
        choices=YES_NO,
    )

    dm_dx_6m = models.CharField(
        verbose_name="Was the initial diagnosis made more than 6 months ago",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    dm_dx_ago = DurationYMDField(
        verbose_name="If 'Yes', estimate how long ago the initial diagnosis was made",
        null=True,
        blank=True,
    )

    dm_complications = models.CharField(
        verbose_name=(
            "Does the patient suffer from any complications of diabetes that are "
            "unmanaged or uncontrolled."
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    htn_dx = models.CharField(
        verbose_name="Has the patient previously been diagnosed with Hypertension",
        max_length=15,
        choices=YES_NO,
    )

    htn_dx_6m = models.CharField(
        verbose_name="Was the initial diagnosis made more than 6 months ago",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    htn_dx_ago = DurationYMDField(
        verbose_name="If 'Yes', estimate how long ago the initial diagnosis was made",
        null=True,
        blank=True,
    )

    htn_complications = models.CharField(
        verbose_name=(
            "Does the patient suffer from any complications of hypertension that are "
            "unmanaged or uncontrolled."
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    excluded_by_bp_history = models.CharField(
        verbose_name=(
            "Has the patient's blood pressure exceeded 180/110 mmHg on more than "
            "one occasion any time in the last 6 months."
        ),
        max_length=15,
        choices=YES_NO,
    )

    excluded_by_gluc_history = models.CharField(
        verbose_name=(
            "Has the patient's blood glucose exceeded 13 mmol/L any time in the last 6 months."
        ),
        max_length=15,
        choices=YES_NO,
    )

    requires_acute_care = models.CharField(
        verbose_name=(
            "Does the patient have any clinical condition that requires health "
            "facility management"
        ),
        max_length=15,
        choices=YES_NO,
    )

    pregnant = models.CharField(
        verbose_name="Is the patient pregnant?", max_length=15, choices=PREG_YES_NO_NA
    )

    def __str__(self):
        return (
            f"{self._meta.verbose_name} {self.screening_identifier} "
            f"{self.age_in_years}{self.gender}"
        )

    def save(self, *args, **kwargs):
        try:
            self.patient_log = PatientLog.objects.get(
                patient_log_identifier=self.patient_log_identifier
            )
        except ObjectDoesNotExist:
            raise SubjectScreeningError(
                f"Invalid patient log identifier. Got `{self.patient_log_identifier}`. "
                "Perhaps catch this in the form."
            )
        if self.patient_log:
            if self.patient_log.willing_to_screen == NO:
                raise SubjectScreeningError(
                    f"Patient '{self.patient_log.patient_log_identifier}' is unwilling to "
                    f"screen. Got reason '{self.patient_log.screening_refusal_reason}' "
                    "Perhaps catch this in the form."
                )
            elif self.patient_log.willing_to_screen == TBD:
                raise SubjectScreeningError(
                    f"Patient '{self.patient_log.patient_log_identifier}' has not indicated "
                    f"willingness to screen. Got `{TBD}`. Perhaps catch this in the form. "
                )
            if self.patient_log.hospital_identifier != self.hospital_identifier:
                raise SubjectScreeningError(
                    "Health facility identifier does not match patient log. "
                    f"Got {self.patient_log.hospital_identifier}!={self.hospital_identifier}. "
                    "Perhaps catch this in the form."
                )
            if self.patient_log.initials != self.initials:
                raise SubjectScreeningError(
                    "Initials do not match patient log. "
                    f"Got {self.patient_log.initials}!={self.initials}. "
                    "Perhaps catch this in the form."
                )
        super().save(*args, **kwargs)

    @property
    def patient_group(self):
        return self.patient_log.patientgroup_set.all().first()

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Subject Screening"
        verbose_name_plural = "Subject Screening"
        constraints = [
            UniqueConstraint(
                fields=["legal_name", "initials"], name="%(app_label)s_%(class)s_legal_n_uniq"
            )
        ]
        indexes = ScreeningModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



# Generated by Django 5.0.8 on 2024-08-26 16:55

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0014_eos"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'intecomm_reports.eos_view' as `report_model` from (SELECT subject_identifier, visit_datetime, timepoint, visit_code, visit_code_sequence, offstudy_reason, offstudy_datetime, site_id, TIMESTAMPDIFF(MONTH, visit_datetime, NOW()) AS m, schedule_status FROM (SELECT b1.subject_identifier, b1.visit_datetime, b1.timepoint, appt.visit_code, appt.visit_code_sequence, appt.site_id, reasons.name AS offstudy_reason, offstudy_datetime, baseline_datetime, sched.schedule_status FROM (SELECT v.subject_identifier, MAX(v.report_datetime) AS visit_datetime, MIN(v.report_datetime) AS baseline_datetime, MAX(appt.timepoint) AS timepoint, eos.offstudy_reason_id, MAX(eos.offstudy_datetime) AS offstudy_datetime FROM intecomm_screening_patientlog AS plog LEFT JOIN intecomm_subject_subjectvisit AS v ON v.subject_identifier = plog.subject_identifier LEFT JOIN edc_appointment_appointment AS appt ON v.appointment_id = appt.id LEFT JOIN intecomm_prn_endofstudy AS eos ON eos.subject_identifier = plog.subject_identifier WHERE NOT plog.group_identifier IS NULL AND timepoint >= 12.0 AND v.visit_code_sequence = 0 GROUP BY v.subject_identifier, eos.offstudy_reason_id UNION SELECT v.subject_identifier, MAX(v.report_datetime) AS visit_datetime, MIN(v.report_datetime) AS baseline_datetime, MAX(appt.timepoint) AS timepoint, eos.offstudy_reason_id, MAX(eos.offstudy_datetime) AS offstudy_datetime FROM intecomm_screening_patientlog AS plog LEFT JOIN intecomm_subject_subjectvisit AS v ON v.subject_identifier = plog.subject_identifier LEFT JOIN edc_appointment_appointment AS appt ON v.appointment_id = appt.id LEFT JOIN intecomm_prn_endofstudy AS eos ON eos.subject_identifier = plog.subject_identifier WHERE NOT plog.group_identifier IS NULL AND timepoint < 12.0 AND v.visit_code_sequence = 0 GROUP BY v.subject_identifier, eos.offstudy_reason_id) AS b1 LEFT JOIN edc_appointment_appointment AS appt ON (b1.subject_identifier = appt.subject_identifier AND b1.timepoint = appt.timepoint) LEFT JOIN intecomm_lists_offstudyreasons AS reasons ON offstudy_reason_id = reasons.id LEFT JOIN edc_visit_schedule_subjectschedulehistory AS sched ON b1.subject_identifier = sched.subject_identifier) AS b2) as A ORDER BY site_id, subject_identifier",
                "eos_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "eos_view", engine="django.db.backends.mysql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'intecomm_reports.eos_view' as report_model from (SELECT subject_identifier, visit_datetime, timepoint, visit_code, visit_code_sequence, offstudy_reason, offstudy_datetime, site_id, TIMESTAMPDIFF(NOW(), visit_datetime, MONTH) AS m, schedule_status FROM (SELECT b1.subject_identifier, b1.visit_datetime, b1.timepoint, appt.visit_code, appt.visit_code_sequence, appt.site_id, reasons.name AS offstudy_reason, offstudy_datetime, baseline_datetime, sched.schedule_status FROM (SELECT v.subject_identifier, MAX(v.report_datetime) AS visit_datetime, MIN(v.report_datetime) AS baseline_datetime, MAX(appt.timepoint) AS timepoint, eos.offstudy_reason_id, MAX(eos.offstudy_datetime) AS offstudy_datetime FROM intecomm_screening_patientlog AS plog LEFT JOIN intecomm_subject_subjectvisit AS v ON v.subject_identifier = plog.subject_identifier LEFT JOIN edc_appointment_appointment AS appt ON v.appointment_id = appt.id LEFT JOIN intecomm_prn_endofstudy AS eos ON eos.subject_identifier = plog.subject_identifier WHERE NOT plog.group_identifier IS NULL AND timepoint >= 12.0 AND v.visit_code_sequence = 0 GROUP BY v.subject_identifier, eos.offstudy_reason_id UNION SELECT v.subject_identifier, MAX(v.report_datetime) AS visit_datetime, MIN(v.report_datetime) AS baseline_datetime, MAX(appt.timepoint) AS timepoint, eos.offstudy_reason_id, MAX(eos.offstudy_datetime) AS offstudy_datetime FROM intecomm_screening_patientlog AS plog LEFT JOIN intecomm_subject_subjectvisit AS v ON v.subject_identifier = plog.subject_identifier LEFT JOIN edc_appointment_appointment AS appt ON v.appointment_id = appt.id LEFT JOIN intecomm_prn_endofstudy AS eos ON eos.subject_identifier = plog.subject_identifier WHERE NOT plog.group_identifier IS NULL AND timepoint < 12.0 AND v.visit_code_sequence = 0 GROUP BY v.subject_identifier, eos.offstudy_reason_id) AS b1 LEFT JOIN edc_appointment_appointment AS appt ON (b1.subject_identifier = appt.subject_identifier AND b1.timepoint = appt.timepoint) LEFT JOIN intecomm_lists_offstudyreasons AS reasons ON offstudy_reason_id = reasons.id LEFT JOIN edc_visit_schedule_subjectschedulehistory AS sched ON b1.subject_identifier = sched.subject_identifier) AS b2) as A ORDER BY site_id, subject_identifier",
                "eos_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "eos_view", engine="django.db.backends.postgresql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'intecomm_reports.eos_view' as report_model from (SELECT subject_identifier, visit_datetime, timepoint, visit_code, visit_code_sequence, offstudy_reason, offstudy_datetime, site_id, TIMESTAMPDIFF(NOW(), visit_datetime, MONTH) AS m, schedule_status FROM (SELECT b1.subject_identifier, b1.visit_datetime, b1.timepoint, appt.visit_code, appt.visit_code_sequence, appt.site_id, reasons.name AS offstudy_reason, offstudy_datetime, baseline_datetime, sched.schedule_status FROM (SELECT v.subject_identifier, MAX(v.report_datetime) AS visit_datetime, MIN(v.report_datetime) AS baseline_datetime, MAX(appt.timepoint) AS timepoint, eos.offstudy_reason_id, MAX(eos.offstudy_datetime) AS offstudy_datetime FROM intecomm_screening_patientlog AS plog LEFT JOIN intecomm_subject_subjectvisit AS v ON v.subject_identifier = plog.subject_identifier LEFT JOIN edc_appointment_appointment AS appt ON v.appointment_id = appt.id LEFT JOIN intecomm_prn_endofstudy AS eos ON eos.subject_identifier = plog.subject_identifier WHERE NOT plog.group_identifier IS NULL AND timepoint >= 12.0 AND v.visit_code_sequence = 0 GROUP BY v.subject_identifier, eos.offstudy_reason_id UNION SELECT v.subject_identifier, MAX(v.report_datetime) AS visit_datetime, MIN(v.report_datetime) AS baseline_datetime, MAX(appt.timepoint) AS timepoint, eos.offstudy_reason_id, MAX(eos.offstudy_datetime) AS offstudy_datetime FROM intecomm_screening_patientlog AS plog LEFT JOIN intecomm_subject_subjectvisit AS v ON v.subject_identifier = plog.subject_identifier LEFT JOIN edc_appointment_appointment AS appt ON v.appointment_id = appt.id LEFT JOIN intecomm_prn_endofstudy AS eos ON eos.subject_identifier = plog.subject_identifier WHERE NOT plog.group_identifier IS NULL AND timepoint < 12.0 AND v.visit_code_sequence = 0 GROUP BY v.subject_identifier, eos.offstudy_reason_id) AS b1 LEFT JOIN edc_appointment_appointment AS appt ON (b1.subject_identifier = appt.subject_identifier AND b1.timepoint = appt.timepoint) LEFT JOIN intecomm_lists_offstudyreasons AS reasons ON offstudy_reason_id = reasons.id LEFT JOIN edc_visit_schedule_subjectschedulehistory AS sched ON b1.subject_identifier = sched.subject_identifier) AS b2) as A ORDER BY site_id, subject_identifier",
                "eos_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "eos_view", engine="django.db.backends.sqlite3"
            ),
            atomic=False,
        ),
    ]



from edc_sites.site import sites

from intecomm_lists.constants import HC_III, HC_IV

from .intecomm_site import IntecommSite

fqdn = "intecomm.clinicedc.org"

ug_language_codes = ["en", "lg", "ry", "sw"]
tz_language_codes = ["en", "sw", "mas"]


sites.register(
    IntecommSite(
        101,
        "kasangati",
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        health_facility_type=HC_IV,
        domain=f"kasangati.ug.{fqdn}",
    ),
    IntecommSite(
        102,
        "kisugu",
        health_facility_type=HC_III,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"kisugu.ug.{fqdn}",
    ),
    IntecommSite(
        103,
        "kiswa",
        health_facility_type=HC_III,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"kiswa.ug.{fqdn}",
    ),
    IntecommSite(
        104,
        "kyazanga",
        health_facility_type=HC_IV,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"kyazanga.ug.{fqdn}",
    ),
    IntecommSite(
        105,
        "mpigi",
        health_facility_type=HC_IV,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"mpigi.ug.{fqdn}",
    ),
    IntecommSite(
        106,
        "namayumba",
        health_facility_type=HC_III,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"namayumba.ug.{fqdn}",
    ),
    IntecommSite(
        107,
        "namulonge",
        health_facility_type=HC_III,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"namulonge.ug.{fqdn}",
    ),
    IntecommSite(
        108,
        "ndejje",
        health_facility_type=HC_IV,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"ndejje.ug.{fqdn}",
    ),
    IntecommSite(
        109,
        "sekiwunga",
        health_facility_type=HC_III,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"sekiwunga.ug.{fqdn}",
    ),
    IntecommSite(
        110,
        "wakiso",
        health_facility_type=HC_IV,
        country_code="ug",
        country="uganda",
        language_codes=ug_language_codes,
        domain=f"wakiso.ug.{fqdn}",
    ),
    IntecommSite(
        201,
        "amana",
        health_facility_type=HC_IV,
        title="Amana Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"amana.tz.{fqdn}",
    ),
    IntecommSite(
        202,
        "bagamoyo",
        health_facility_type=HC_IV,
        title="Bagamoyo Regional Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"bagamoyo.tz.{fqdn}",
    ),
    IntecommSite(
        203,
        "rugambwa",
        health_facility_type=HC_IV,
        title="Cardinal Rugambwa Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"rugambwa.tz.{fqdn}",
    ),
    IntecommSite(
        204,
        "hindu_mandal",
        health_facility_type=HC_IV,
        title="Hindu Mandal Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"hindu-mandal.tz.{fqdn}",
    ),
    IntecommSite(
        205,
        "kisarawe",
        health_facility_type=HC_IV,
        title="Kisarawe District Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"kisarawe.tz.{fqdn}",
    ),
    IntecommSite(
        206,
        "mbagala",
        health_facility_type=HC_III,
        title="Mbagala Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"mbagala.tz.{fqdn}",
    ),
    IntecommSite(
        207,
        "mnazi_moja",
        health_facility_type=HC_III,
        title="Mnazi Moja Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"mnazi.tz.{fqdn}",
    ),
    IntecommSite(
        208,
        "mwananyamala",
        health_facility_type=HC_IV,
        title="Mwananyamala Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"mwananyamala.tz.{fqdn}",
    ),
    IntecommSite(
        209,
        "sinza",
        health_facility_type=HC_III,
        title="Sinza Hospital",
        country_code="tz",
        country="tanzania",
        language_codes=tz_language_codes,
        domain=f"sinza.tz.{fqdn}",
    ),
    IntecommSite(
        210,
        "temeke",
        health_facility_type=HC_IV,
        title="Temeke Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=tz_language_codes,
        domain=f"temeke.tz.{fqdn}",
    ),
)



# Generated by Django 4.1.2 on 2022-11-26 03:40

from django.db import migrations
import edc_sites.models
import intecomm_group.models.patient_group


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_group", "0009_auto_20221126_0358"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="patientgroup",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", intecomm_group.models.patient_group.PatientGroupManager()),
            ],
        ),
    ]



from __future__ import annotations

from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..forms import OffScheduleInteForm
from ..models import OffScheduleInte
from .modeladmin_mixins import OffScheduleAdmin


@admin.register(OffScheduleInte, site=intecomm_prn_admin)
class OffScheduleInteAdmin(OffScheduleAdmin):
    form = OffScheduleInteForm

    # TODO: remove with Django > 4.2.5
    def get_list_filter(self, request) -> tuple[str]:
        list_filter = super().get_list_filter(request)
        self.list_filter = list_filter
        return list_filter



# Generated by Django 4.2.11 on 2024-06-11 21:54
from django.db import migrations
from edc_model.utils import InvalidFormat, duration_hm_to_timedelta
from tqdm import tqdm


def update_durations(apps, schema_editor):
    for model, fldname in [
        ("intecomm_subject.dminitialreview", "glucose_fasting_duration_delta"),
        ("intecomm_subject.dmreview", "glucose_fasting_duration_delta"),
        ("intecomm_subject.glucose", "glucose_fasting_duration_delta"),
    ]:
        print(model)
        model_cls = apps.get_model(model)
        qs = model_cls.objects.all()
        total = qs.count()
        for obj in tqdm(qs, total=total):
            if duration_text := obj.glucose_fasting_duration_str:
                try:
                    obj.glucose_fasting_duration_delta = duration_hm_to_timedelta(
                        duration_text
                    )
                except (TypeError, InvalidFormat):
                    print(obj)
                else:
                    obj.save_base(update_fields=[fldname])
    for model, fldname in [
        ("intecomm_subject.careseekinga", "travel_tdelta"),
        ("intecomm_subject.careseekingb", "travel_tdelta"),
    ]:
        print(model)
        model_cls = apps.get_model(model)
        qs = model_cls.objects.all()
        total = qs.count()
        for obj in tqdm(qs, total=total):
            if duration_text := obj.travel_duration:
                try:
                    obj.travel_tdelta = duration_hm_to_timedelta(duration_text)
                except (TypeError, InvalidFormat):
                    print(obj)
                else:
                    obj.save_base(update_fields=[fldname])


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0185_rename_currentconditions_diagnoses_and_more"),
    ]

    operations = [migrations.RunPython(update_durations)]



from django import forms
from edc_constants.constants import DM, HIV, HTN, NO, OTHER, YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfSingletonModelFormMixin
from edc_form_validators import INVALID_ERROR, FormValidator

from ...constants import ADULT, CHILD, MAIN_EARNER, STUDY_VISIT
from ...models import CareseekingA
from ..mixins import CrfModelFormMixin


class CareseekingAFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self) -> None:
        care_seeking_a = CareseekingA.objects.filter(
            subject_visit__subject_identifier=self.related_visit.subject_identifier
        ).exclude(subject_visit_id=self.related_visit.id)
        if care_seeking_a.count() > 0:
            raise forms.ValidationError(
                f"This form has already been submitted. See {care_seeking_a[0].subject_visit}."
            )
        self.m2m_single_selection_if(STUDY_VISIT, m2m_field="care_visit_reason")

        self.m2m_other_specify(
            m2m_field="care_visit_reason", field_other="care_visit_reason_other"
        )

        # A3: Medication costs for today's visit
        self.m2m_required_if(YES, field="med_prescribed", m2m_field="med_conditions")
        self.m2m_other_specify(m2m_field="med_conditions", field_other="med_conditions_other")
        self.applicable_if(YES, field="med_prescribed", field_applicable="med_collected")

        self.applicable_if(
            NO, field="med_collected", field_applicable="med_not_collected_reason"
        )
        self.validate_other_specify(
            field="med_not_collected_reason",
            other_specify_field="med_not_collected_reason_other",
        )

        self.required_if(
            YES,
            field="med_collected",
            field_required="med_cost_tot",
            field_required_evaluate_as_int=True,
        )

        med_conditions = [o.name for o in self.cleaned_data.get("med_conditions")]
        med_cost_tot = self.cleaned_data.get("med_cost_tot")
        self.required_if(
            YES,
            field="med_collected",
            field_required="med_cost_tot",
            field_required_evaluate_as_int=True,
        )
        self.required_if_true(
            HIV in med_conditions and med_cost_tot is not None and med_cost_tot > 0,
            field_required="med_cost_hiv",
        )
        self.required_if_true(
            HTN in med_conditions and med_cost_tot is not None and med_cost_tot > 0,
            field_required="med_cost_htn",
        )
        self.required_if_true(
            DM in med_conditions and med_cost_tot is not None and med_cost_tot > 0,
            field_required="med_cost_dm",
        )
        self.required_if_true(
            OTHER in med_conditions and med_cost_tot is not None and med_cost_tot > 0,
            field_required="med_cost_other",
        )
        self.applicable_if(
            YES, field="med_collected", field_applicable="med_collected_location"
        )
        self.validate_other_specify(
            field="med_collected_location", other_specify_field="med_collected_location_other"
        )

        # A4: Diagnostic costs for today's visit
        self.applicable_if(YES, field="tests_requested", field_applicable="tests_done")
        self.applicable_if(NO, field="tests_done", field_applicable="tests_not_done_reason")
        self.validate_other_specify(
            field="tests_not_done_reason", other_specify_field="tests_not_done_other"
        )
        self.required_if(
            YES,
            field="tests_done",
            field_required="tests_cost",
            field_required_evaluate_as_int=True,
        )

        self.validate_other_specify(
            field="missed_activities", other_specify_field="missed_activities_other"
        )

        self.applicable_if(YES, field="referral", field_applicable="referral_type")
        self.applicable_if(YES, field="referral", field_applicable="referral_facility")

        # accompany
        self.required_if(
            ADULT,
            CHILD,
            MAIN_EARNER,
            field="accompany",
            field_required="accompany_num",
            field_required_evaluate_as_int=True,
        )
        self.applicable_if_true(
            self.cleaned_data.get("accompany_num")
            and self.cleaned_data.get("accompany_num") > 0,
            field_applicable="accompany_wait",
        )
        self.applicable_if_true(
            self.cleaned_data.get("accompany_num")
            and self.cleaned_data.get("accompany_num") > 0,
            field_applicable="accompany_alt",
        )
        self.validate_other_specify(
            field="accompany_alt", other_specify_field="accompany_alt_other"
        )

        # A8: Your expenses for today's visit / money_sources
        money_sources = [o.name for o in self.cleaned_data.get("money_sources") or []]
        if len(money_sources) > 3:
            self.raise_validation_error(
                {"money_sources": "Please limit to no more than 3 selections"}, INVALID_ERROR
            )
        self.m2m_other_specify(m2m_field="money_sources", field_other="money_sources_other")

        if self.cleaned_data.get("money_source_main") and self.cleaned_data.get(
            "money_source_main"
        ) not in [o.name for o in self.cleaned_data.get("money_sources") or []]:
            self.raise_validation_error(
                {"money_source_main": "Response not found among responses given above"},
                INVALID_ERROR,
            )


class CareseekingAForm(
    CrfSingletonModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = CareseekingAFormValidator

    class Meta:
        model = CareseekingA
        fields = "__all__"
